import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;

/**
 * A Counter class that allows you to display a numerical value on screen.
 * 
 * The Counter is an actor, so you will need to create it, and then add it to
 * the world in Greenfoot.  If you keep a reference to the Counter then you
 * can adjust its value.  Here's an example of a world class that
 * displays a counter with the number of act cycles that have occurred:
 * 
 * <pre>
 * class CountingWorld
 * {
 *     private Counter actCounter;
 *     
 *     public CountingWorld()
 *     {
 *         super(600, 400, 1);
 *         actCounter = new Counter("Act Cycles: ");
 *         addObject(actCounter, 100, 100);
 *     }
 *     
 *     public void act()
 *     {
 *         actCounter.setValue(actCounter.getValue() + 1);
 *     }
 * }
 * </pre>
 * 
 * @author Neil Brown and Michael Kölling 
 * Modified by Kenzie Togami
 * @version 1.0
 */
public class Player extends Actor
{
    static enum Direction {
        H("h"), 

        V("v");

        String ts = "";
        Direction(String toString) {
            ts = toString;
        }

        public String toString() {
            return ts;
        }
    }
    public static final Color transparent = new Color(0,0,0,0);
    private GreenfootImage background;
    private int value;
    private int target;
    private String prefix;
    public Color drawColor = Color.BLACK;
    Blank b;
    Direction dir;

    public Player()
    {
        this("");
    }

    /**
     * Create a new counter, initialised to 0.
     */
    public Player(String prefix)
    {
        background = getImage();  // get image from class
        value = 0;
        target = 0;
        this.prefix = prefix;
        updateImage();
    }

    /**
     * Animate the display to count up (or down) to the current target value.
     * Also chooses the player direction!
     */
    public void act() 
    {
        if (value < target) {
            value++;
            updateImage();
        }
        else if (value > target) {
            value--;
            updateImage();
        }
        String key = Blank.key;
        if (b == null) {
            b = (Blank) getWorld();
        }
        if(Greenfoot.mouseClicked(this) && b.curr == null)
        {
            b.setAsCurr(this);
        }
        else if("v".equals(key) && b.noDir && b.curr != null && b.curr != this)
        {
            setDir(Direction.V);
        }
        else if("h".equals(key) && b.noDir && b.curr != null && b.curr != this)
        {
            setDir(Direction.H);
        }
    }

    public void setDir(Direction d)
    {
        setPrefix(d + " " + prefix);
        dir = d;
        Direction otherdir;
        switch(dir)
        {
            case V: otherdir = Direction.H; break;
            case H: otherdir = Direction.V; break;
            default: throw new IllegalStateException("No direction?");
        }
        Player curr = b.curr; // using direct access so that nulls are caught
        if (curr == null) {
            throw new IllegalStateException("null player");
        }
        curr.setPrefix(otherdir + " " + curr.prefix);
        curr.dir = otherdir;
        b.noDir = false;
    }

    /**
     * Add a new score to the current counter value.  This will animate
     * the counter over consecutive frames until it reaches the new value.
     */
    public void add(int score)
    {
        target += score;
    }

    /**
     * Return the current counter value.
     */
    public int getValue()
    {
        return target;
    }

    /**
     * Set a new counter value.  This will not animate the counter.
     */
    public void setValue(int newValue)
    {
        target = newValue;
        value = newValue;
        updateImage();
    }

    /**
     * Sets a text prefix that should be displayed before
     * the counter value (e.g. "Score: ").
     */
    public void setPrefix(String prefix)
    {
        this.prefix = prefix;
        updateImage();
    }

    public void updateImage()
    {
        updateImage(drawColor);
    }

    /**
     * Update the image on screen to show the current value.
     */
    public void updateImage(Color c)
    {
        setImage(new GreenfootImage(toString(), 16, c, transparent));
    }

    public String toString() {
        return prefix + value;
    }
}
